//Phong Reflection Model Vertex Shader
#version 430
 
layout(location = 0) in vec3 pos; // local space
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 texCoords;

// matrices
uniform mat4 m_pvm;
uniform mat4 m_vm;
uniform mat4 m_m;
uniform mat3 m_normal;

// DoF
uniform float focalDistance, focalRange;

out Data{
	out vec3 pos; // in world space
	out vec3 normal; // normalized and in world space
	out vec2 texCoords;	
	out float blur; // the amount of blur depending on the distance to the eye
} dataOut;
 
void main() {
	gl_Position = m_pvm*vec4(pos, 1);
	dataOut.pos = vec3(m_m*vec4(pos, 1));
	dataOut.normal = normalize(m_normal * normal);
	dataOut.texCoords = texCoords;
	
	// calculate amount of blur
	vec3 posView = vec3(m_vm * vec4(pos, 1)); // position in view space
	dataOut.blur = clamp(abs(-posView.z - focalDistance) / focalRange, 0.0, 1.0);
}